﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HidLibrary;

namespace MonoGameDemo.InputManagement
{
    public class InputManager
    {
        public static InputManager Instance;

        Dictionary<String, InputElement> Elements = new Dictionary<string, InputElement>();
        List<HIDDeviceHandler> handlers = new List<HIDDeviceHandler>();

        public InputManager()
        {
            Instance = this;
        }

        #region Getters
        public float GetFloat(string device, string name)
        {
            float res = 0;
            if (Elements.ContainsKey(device))
            {
                if (Elements[device].Floats.ContainsKey(name))
                {
                    res = Elements[device].Floats[name];
                }
            }
            return res;
        }

        public int GetInt(string device, string name)
        {
            int res = 0;
            if (Elements.ContainsKey(device))
            {
                if (Elements[device].Ints.ContainsKey(name))
                {
                    res = Elements[device].Ints[name];
                }
            }
            return res;
        }

        public bool GetBool(string device, string name)
        {
            bool res = false;
            if (Elements.ContainsKey(device))
            {
                if (Elements[device].Bools.ContainsKey(name))
                {
                    res = Elements[device].Bools[name];
                }
            }
            return res;
        }

        public HiHat GetHiHat(string device, string name)
        {
            HiHat res = null;
            if (Elements.ContainsKey(device))
            {
                if (Elements[device].Hats.ContainsKey(name))
                {
                    res = Elements[device].Hats[name];
                }
            }
            return res;
        }
        #endregion

        #region Setters
        public void SetInt(String device, string name, int value)
        {
            if (Elements.ContainsKey(device))
            {
                if (Elements[device].Ints.ContainsKey(name))
                {
                    Elements[device].Ints[name] = value;
                }
            }
        }
        public void SetFloat(String device, string name, float value)
        {
            if (Elements.ContainsKey(device))
            {
                if (Elements[device].Floats.ContainsKey(name))
                {
                    Elements[device].Floats[name] = value;
                }
            }
        }
        public void SetBool(String device, string name, bool value)
        {
            if (Elements.ContainsKey(device))
            {
                if (Elements[device].Bools.ContainsKey(name))
                {
                    Elements[device].Bools[name] = value;
                }
            }
        }
        public void SetHiHat(String device, string name, int value)
        {
            if (Elements.ContainsKey(device))
            {
                if (Elements[device].Hats.ContainsKey(name))
                {
                    Elements[device].Hats[name].FromInt(value);
                }
            }
        }
        #endregion


        #region Device setup
        public void RegisterDeviceFloat(string device, string name)
        {
            Elements[device].Floats.Add(name, 0);
        }
        public void RegisterDeviceInt(string device, string name)
        {
            Elements[device].Ints.Add(name, 0);
        }
        public void RegisterDeviceBool(string device, string name)
        {
            Elements[device].Bools.Add(name, false);
        }
        public void RegisterDeviceHat(string device, string name)
        {
            Elements[device].Hats.Add(name, new HiHat());
        }
        #endregion

        #region Device management
        public void Connect(HidDevice device)
        {
            InputElement e = new InputElement();
            Elements.Add(device.Description, e);
            HIDDeviceHandler hid = new HIDDeviceHandler(device);
            handlers.Add(hid);
        }
        #endregion

        #region Static accessors
        public static float GetNamedFloat(string device, string input)
        {
            return Instance.GetFloat(device, input);
        }

        public static int GetNamedInt(string device, string input)
        {
            return Instance.GetInt(device, input);
        }

        public static bool GetNamedBool(string device, string input)
        {
            return Instance.GetBool(device, input);
        }

        public static HiHat GetNamedHiHat(string device, string name)
        {
            return Instance.GetHiHat(device, name);
        }

        public static void SetNamedInt(string device, string name, int value)
        {
            Instance.SetInt(device, name, value);
        }
        public static void SetNamedFloat(string device, string name, float value)
        {
            Instance.SetFloat(device, name, value);
        }
        public static void SetNamedBool(string device, string name, bool value)
        {
            Instance.SetBool(device, name, value);
        }
        public static void SetNamedHiHat(string device, string name, int value)
        {
            Instance.SetHiHat(device, name, value);
        }



        public static void ConnectToDevice(HidDevice device)
        {
            Instance.Connect(device);
        }

        public static void RegisterFloat(string device, string name)
        {
            Instance.RegisterDeviceFloat(device, name);
        }
        public static void RegisterInt(string device, string name)
        {
            Instance.RegisterDeviceInt(device, name);
        }
        public static void RegisterBool(string device, string name)
        {
            Instance.RegisterDeviceBool(device, name);
        }
        public static void RegisterHiHat(string device, string name)
        {
            Instance.RegisterDeviceHat(device, name);
        }
        #endregion
    }
}
